import json

try:
    import bpy
    import mathutils
    from . import scene
    from .. import utils
    from .. import config
    from ..tools import bouncecurve as bc
    from ..bouncecurve import bcutils
    from .gen_curve import GenerateBounceCurveOperator
    from .gen_score import GenerateScoreOperator
    from .camera import GenerateCameraOperator
    from .rectify import RectifyOffsetOperator
except ModuleNotFoundError as e:
    import scene
    import utils
    import config
    import bcutils
    from tools import bouncecurve as bc
    from gen_curve import GenerateBounceCurveOperator
    from gen_score import GenerateScoreOperator
    from camera import GenerateCameraOperator
    from rectify import RectifyOffsetOperator


def calc_x_to_time(parts):
    time_to_x = {}
    for part in parts:
        times = [t.get('onset_time') for t in part.get('times', [])]
        if len(times) < 1:
            continue
        note_paths = part.get('note_paths', [])
        for i, t in enumerate(times):
            if t in time_to_x.keys():
                continue
            time_to_x[t] = note_paths[0][i][0]
    return {x: t for t, x in time_to_x.items()}


class CopyAnimationOperator(bpy.types.Operator):
    bl_idname = "object.bc_copy_animation"
    bl_label = "复制动画到选中对象"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        data = context.scene.bc_score_data
        try:
            j = json.loads(data)
        except:
            self.report({'ERROR'}, '乐谱数据丢失或未生成乐谱，请重新生成乐谱')
            return {'FINISHED'}
        parts = j.get('parts', [])
        if len(parts) < 1:
            self.report({'INFO'}, '乐谱数据异常或者未生成乐谱')
            return {'FINISHED'}
        obj_ani_source = context.scene.bc_obj_animation_source
        if obj_ani_source is None:
            self.report({'ERROR'}, '请选择动复制动画对象')
            return {'FINISHED'}
        selected_objs = bpy.context.selected_objects
        if selected_objs is None or len(selected_objs) == 0:
            self.report({'ERROR'}, '请选中要赋予动画的对象')
            return {'FINISHED'}
        if obj_ani_source.animation_data is None:
            self.report({'ERROR'}, f'对象{obj_ani_source.name}没有动画数据')
            return {'FINISHED'}
        if context.scene.bc_use_same_material:
            for obj in selected_objs:
                obj.active_material = obj_ani_source.active_material
        x_to_time = calc_x_to_time(parts)
        x_to_frame = {x: round(t * bpy.context.scene.render.fps) for x, t in x_to_time.items()}
        rel_frame = context.scene.bc_relative_frame
        for obj in selected_objs:
            if obj == obj_ani_source:
                continue
            for prop in obj_ani_source.keys():
                obj[prop] = obj_ani_source[prop]
            obj.animation_data_clear()
            obj.animation_data_create()
            obj.animation_data.action = obj_ani_source.animation_data.action.copy()
            for fcurve in obj.animation_data.action.fcurves:
                for keyframe in fcurve.keyframe_points:
                    target_frame = bcutils.calc_frame(obj.location[0], x_to_frame)
                    keyframe.co[0] = keyframe.co[0] - rel_frame + target_frame
                    if keyframe.handle_left is not None:
                        keyframe.handle_left[0] = keyframe.handle_left[0] - rel_frame + target_frame
                    if keyframe.handle_right is not None:
                        keyframe.handle_right[0] = keyframe.handle_right[0] - rel_frame + target_frame
            # obj.animation_data.action.update()
        return {'FINISHED'}


class BounceCurvePanel(bpy.types.Panel):
    bl_label = "跳动曲线"
    bl_idname = "TOOL_PT_mercury_client_panel_bounce_curve"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = 'world'

    @classmethod
    def poll(cls, context):
        return True

    def draw(self, context):
        layout = self.layout
        layout.use_property_split = True
        # layout.use_property_decorate = False
        layout.label(text="全局配置")
        layout.row().prop(context.scene, "mc_expire_tips", placeholder='无需填写')
        layout.row().prop(context.scene, "mc_auth_key")
        layout.row().prop(context.scene, "mc_auth_secret")
        layout.separator()
        layout.prop(context.scene, "bc_midi")
        layout.prop(context.scene, "bc_import_svg_as_gspencil")
        layout.operator(GenerateScoreOperator.bl_idname, text="生成乐谱")
        layout.separator()
        layout.label(text="动画配置")
        # layout.prop(context.scene, "bc_obj_svg_score")
        layout.prop(context.scene, "bc_obj_bounce")
        layout.prop(context.scene, "bc_bounce_factor")
        layout.prop(context.scene, "bc_bounce_max_height")
        layout.operator(GenerateBounceCurveOperator.bl_idname, text="生成弹跳曲线/动画")
        layout.separator()
        layout.prop(context.scene, "bc_obj_animation_source")
        layout.prop(context.scene, "bc_relative_frame")
        layout.prop(context.scene, "bc_use_same_material")
        layout.operator(CopyAnimationOperator.bl_idname, text="复制动画到选中对象")
        layout.separator()
        layout.prop(context.scene, "bc_animation_delay")
        layout.operator(RectifyOffsetOperator.bl_idname, text='偏移校正')
        layout.separator()
        layout.prop(context.scene, "bc_camera")
        layout.operator(GenerateCameraOperator.bl_idname, text="生成摄像机动画")
